// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of passive_data;

/// Convert a dynamic type to a PDS.
dynamic pdsFromDynamic(final dynamic value, [bool retry = true]) {
  // Check for num, bool, null, String
  if (value is num || value is bool || value == null || value is String) {
    return value;
  } else {
    // Check for List, Set, Map<String, Map<int
    if (value is List) {
      return pdsFromList(value);
    } else if (value is Set) {
      return pdsFromSet(value);
    } else if (value is Map) {
      return pdsFromMap(value);
    } else if (retry) {
      // Fallback to `.toPds()`
      try {
        var passive = value.toPds();
        return pdsFromDynamic(passive, false);
      } catch (e) {
        throw new ArgumentError.value(
            value, 'value', 'cannot be converted to a passive data structure');
      }
    } else {
      throw new ArgumentError.value(
          value, 'value', 'cannot be converted to a passive data structure');
    }
  }
}

/// Convert a List to a PDS.
List pdsFromList(final List list) {
  return new List.generate(list.length, (int i) => pdsFromDynamic(list[i]),
      growable: false);
}

/// Convert a Set to a PDS.
List pdsFromSet(final Set theset) {
  return pdsFromList(theset.toList(growable: false));
}

/// Convert a String indexed Map to a PDS.
Map<String, dynamic> pdsFromMap(final Map map) {
  return new Map<String, dynamic>.fromIterable(map.keys,
      key: (key) => key is String ? key : key.toString(),
      value: (key) => pdsFromDynamic(map[key]));
}

/// Convert a PDS List back to a typed List.
List pdsToList(final List list, [dynamic convert(dynamic item) = null]) {
  return new List.generate(
      list.length, (int i) => convert == null ? list[i] : convert(list[i]));
}

/// Convert a PDS List back to a Set.
Set pdsToSet(final List list, [dynamic convert(dynamic item) = null]) {
  return pdsToList(list, convert).toSet();
}

/// Convert a PDS Map back to a typed String Map.
Map<String, dynamic> pdsToStringMap(final Map<String, dynamic> map,
    [dynamic convert(dynamic item) = null]) {
  return new Map<String, dynamic>.fromIterable(map.keys,
      key: (String key) => key,
      value: (String key) => convert == null ? map[key] : convert(map[key]));
}

/// Convert a PDS Map back to a typed int Map.
Map<int, dynamic> pdsToIntMap(final Map map,
    [dynamic convert(dynamic item) = null]) {
  return new Map<int, dynamic>.fromIterable(map.keys,
      key: (String key) => int.parse(key),
      value: (String key) => convert == null ? map[key] : convert(map[key]));
}
