// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library passive_data.test.generate_code_test;

import 'package:test/test.dart';
import 'package:passive_data/passive_data.dart';

/// 'd' is used as data access name.
const Map<String, String> typeToPds = const {
  'String': 'd',
  'List<Point>': 'pdsFromList(d)',
  'Set<Point>': 'pdsFromSet(d)',
  'Map<String, int>': 'd',
  'Map<int, String>': 'pdsFromMap(d)',
  'Map<int, Point>': 'pdsFromMap(d)',
  'Map<String, Point>': 'pdsFromMap(d)',
  'Point': 'd.toPds()'
};

/// 'd' is used as data access name. Cast is set to false initially.
/// All spaces and line endings are stripped.
Map<List<String>, String> pdsToType = {
  ['dynamic']: 'd',
  ['String']: 'd as String',
  ['Set<String>', 'List<String>']: 'new Set<String>.from(d)',
  [
    'List<Point>',
    'List<String>'
  ]: 'pdsToList(d as List, (String item) => new Point.fromPds(item)) as List<Point>',
  [
    'Map<String, Map<int, dynamic>>',
    'Map<String, Map<String, dynamic>>'
  ]: 'pdsToStringMap(d as Map<String, dynamic>, (Map<String, dynamic> item) => pdsToIntMap(item)) as Map<String, Map<int, dynamic>>',
  [
    'Map<String, Point>',
    'Map<String, Map<String, dynamic>>'
  ]: 'pdsToStringMap(d as Map<String, dynamic>, (Map<String, dynamic> item) => new Point.fromPds(new Map<String, dynamic>.from(item))) as Map<String, Point>',
  [
    'Map<int, Point>',
    'Map<String, Map<String, dynamic>>'
  ]: 'pdsToIntMap(d as Map<String, dynamic>, (Map<String, dynamic> item) => new Point.fromPds(new Map<String, dynamic>.from(item))) as Map<int, Point>',
  ['Point', 'Map<String, dynamic>']:
      'new Point.fromPds(new Map<String, dynamic>.from(d))',
  [
    'List<Set<MyObject>>',
    'List<List<int>>'
  ]: 'pdsToList(d as List, (List<int> item) => pdsToSet(item, (int item) => new MyObject.fromPds(item))) as List<Set<MyObject>>'
};

/// All throw an ArgumentError.
List<List<String>> pdsToTypeExceptions = [
  ['Map<int, Point>', 'Set<int>'],
  ['Map<int, Point>', ''],
  ['List<Point>', 'int'],
  ['List<Point>', '']
];

void main() {
  test('Test generateTypeToPdsCode', () {
    typeToPds.forEach((String type, String expected) {
      expect(_stripSpacesAndLf(generateTypeToPdsCode(type, 'd')),
          equals(_stripSpacesAndLf(expected)));
    });
  });

  test('Test generatePdsToTypeCode', () {
    pdsToType.forEach((List<String> types, String expected) {
      expect(
          _stripSpacesAndLf(generatePdsToTypeCode(
              types[0], 'd', types.length > 1 ? types[1] : '')),
          equals(_stripSpacesAndLf(expected)));
    });
  });

  test('Test generatePdsToTypeCode exceptions', () {
    pdsToTypeExceptions.forEach((List<String> types) {
      expect(() => generatePdsToTypeCode(types[0], 'd', types[1]),
          throwsArgumentError);
    });
  });
}

/// Remove spaces and line feeds from the given string.
String _stripSpacesAndLf(String str) {
  str = str.replaceAll(' ', '');
  str = str.replaceAll('\n', '');
  return str;
}
